<?php

namespace Pinpoint\PackingSlips\Model\Pdf;

use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class GenerateSlip
 * @package Pinpoint\PackingSlips\Model\Pdf
 */
class GenerateSlip extends \Magento\Framework\Model\AbstractModel
{
    /**
     * @var \Dompdf\Dompdf
     */
    protected $domPdf;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterfaceFactory
     */
    protected $productRepository;

    /**
     * @var \Magento\Framework\Pricing\Helper\Data
     */
    protected $currencyHelper;

    /**
     * @var \Magento\Framework\View\LayoutFactory
     */
    protected $layoutFactory;

    /**
     * GenerateSlip constructor.
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Pricing\Helper\Data $currencyHelper
     * @param \Magento\Catalog\Api\ProductRepositoryInterfaceFactory $productRepositoryFactory
     * @param \Magento\Framework\View\LayoutFactory $layoutFactory
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Pricing\Helper\Data $currencyHelper,
        \Magento\Catalog\Api\ProductRepositoryInterfaceFactory $productRepositoryFactory,
        \Magento\Framework\View\LayoutFactory $layoutFactory,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $mediaDir = DIRECTORY_SEPARATOR . trim($filesystem->getDirectoryRead(DirectoryList::MEDIA)->getAbsolutePath(), '/');
        $fontFolder = $mediaDir . DIRECTORY_SEPARATOR . 'dompdf_fonts';
        $fontCacheFolder = $mediaDir . DIRECTORY_SEPARATOR . 'dompdf_fonts_cache';

        if (!is_dir($fontFolder)) {
            mkdir($fontFolder, 0777, true);
        }

        if (!is_dir($fontCacheFolder)) {
            mkdir($fontCacheFolder, 0777, true);
        }

        $this->domPdf = new \Dompdf\Dompdf([
            'enable_remote' => true,
            'DOMPDF_ENABLE_CSS_FLOAT' => true,
            'defaultFont' => 'Helvetica',
            'fontDir' => $fontFolder,
            'fontCache' => $fontCacheFolder
        ]);

        $this->orderRepository = $orderRepository;
        $this->productRepository = $productRepositoryFactory;
        $this->currencyHelper = $currencyHelper;
        $this->layoutFactory = $layoutFactory;

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * @param $orderId
     * @return \Dompdf\Dompdf
     */
    public function generateSlip($orderId)
    {
        $slip = $this->domPdf;
        $order = $this->orderRepository->get($orderId);

        $pdfData = [
            'order' => [
                'id' => $orderId,
                'date_purchased' => $order->getCreatedAt(),
                'payment_method' => $order->getPayment()->getData('additional_information')['method_title'],
                'discount' => $order->getDiscountAmount(),
                'barcode' => $orderId
            ],
            'customer' => [
                'customer_name' => sprintf('%s %s', $order->getShippingAddress()->getFirstname(), $order->getShippingAddress()->getLastname()),
                'billing_address' => $order->getBillingAddress(),
                'delivery_address' => $order->getShippingAddress(),
            ],
            'order_items' => $order->getItems()
        ];

        $requiredMachines = [];

        /** @var \Magento\Sales\Api\Data\OrderItemInterface $item */
        foreach ($pdfData['order_items'] as $item) {
            $product = $this->productRepository->create()->getById($item->getProductId());
            $pdfData['products'][$item->getProductId()] = $product;
            $requiredMachines[] = $productMachines = $product->getResource()->getAttribute('required_machines')->getFrontend()->getValue($product);
        }


        $pdfData['order']['required_machines'] = rtrim(implode(',', $requiredMachines), ',');
        $pdfData['order']['id'] = str_pad($pdfData['order']['id'], 9, 0, STR_PAD_LEFT);

        $slip->loadHtml($this->getPdfTemplate($pdfData));
        $slip->render();

        return $slip;
    }

    /**
     * @param $params
     * @return mixed
     */
    protected function getPdfTemplate($params)
    {
        return $this->layoutFactory->create()
            ->createBlock('Pinpoint\PackingSlips\Block\Adminhtml\Pdf\PackingSlip')
            ->setData($params)
            ->toHtml()
        ;
    }
}